function [L, Mu, Kappa] = vmf_kmeanspp(X, k, flags)
% KMEANS Cluster data on the unit-sphere using the k-means++ algorithm.
%   [L,Mu,Kappa] = vmf_meanspp(X,k) produces a 1-by-size(X,2) vector L with
%   one class label per colum in X, a size(X,1)-by-k matrix Mu containing 
%   the centers corresponding to each class and a size(X,1)-by-k matrix 
%   Kappa contining the Kappas corresponding to each class.
%
%   Code written by Reshad Hosseini (reshad.hosseini@ut.ac.ir)
%
%   References:
%   [1] J. B. MacQueen, "Some Methods for Classification and Analysis of 
%       MultiVariate Observations", in Proc. of the fifth Berkeley
%       Symposium on Mathematical Statistics and Probability, L. M. L. Cam
%       and J. Neyman, eds., vol. 1, UC Press, 1967, pp. 281-297.
%   [2] D. Arthur and S. Vassilvitskii, "k-means++: The Advantages of
%       Careful Seeding", Technical Report 2006-13, Stanford InfoLab, 2006.

if nargin < 3
    flags = struct;
end

% Using hard or soft clustering
if isfield(flags,'hardcluster')
    hardcluster = flags.hardcluster;
else
    hardcluster = true; %false
end

% Equal or non-equal Kappas
if isfield(flags,'equalKappa')
    equalKappa = flags.equalKappa;
else
    equalKappa = true; 
end

% Equal or non-equal Ps
if isfield(flags,'equalP')
    equalP = flags.equalP;
else
    equalP = true; %false
end

% Using Kmeanspp initialization or random initialization
if isfield(flags,'kmeanspp')
    kmeanspp = flags.kmeanspp;
else
    kmeanspp = true; 
end

% Not Allowing Kappa to be greatet than boundKappaVal, yes or no
if isfield(flags,'boundKappa')
    boundKappa = flags.boundKappa;
else
    boundKappa = false; 
end
boundKappaVal = 1e4;

L1 = 0;
L = [];
countLabel = zeros(1,k);

[Dim, N] = size(X);
Nu = 0.5*Dim - 1;

while length(unique(L)) ~= k
    if kmeanspp
        % K-means++ initialization
        Mu = X(:, ceil(rand * N));
        Dt = (2 * Mu).' *X;
        L = ones(1, N);
        for i = 2:k
            %fprintf(':');
            D = cumsum(2 - Dt); 
            if D(end) == 0, Mu(:,i:k) = X(:,ones(1,k-i+1)); return; end
            index = find(rand < D/D(end), 1);Mu(:, i) = X(:, index);
            Dtold = Dt;
            Dt = max(Dtold, (2 * Mu(:,i)).'*X);
            L(Dt ~= Dtold) = i;
        end
        %fprintf('\n');
    else
        % Random Initialization
        L = ceil(rand(1,N) * k);
    end
    % The k-means algorithm.
    Kappa = [];
    weight = [];
    P = 1;
    while any(L ~= L1)
        %fprintf('-');
        L1 = L;
        if hardcluster || isempty(weight)
            for i = 1:k, 
                l = L==i; 
                Mu(:,i) = sum(X(:,l),2); 
                countLabel(i) = sum(l);
            end
        else
            Mu = X * weight;
        end
        Rbar = sqrt(sum(Mu.^2));
        Mu = Mu./(ones (Dim,1) * Rbar); 

        if hardcluster && equalKappa && equalP
            [unused, L] = max(Mu.'*X,[],1); %#ok<ASGLU>
        else
            if equalKappa
                Rbar = sum(Rbar) /N;
            elseif isempty(weight) || hardcluster
                Rbar = Rbar ./ countLabel;
            else
                Rbar = Rbar ./ sum(weight, 1);
            end
            Kappa = (Rbar*Dim - Rbar.^3)./(1-Rbar.^2);
            if boundKappa
                % Bounding Kappa
                Kappa (Kappa > boundKappaVal) = boundKappaVal;
            end
            if hardcluster && equalKappa
                LogNormalize  = 0;
            else
                % Using Log-likelihood of vMF (Remove constant terms)
                % For Log of the Bessel use Eqn 9.7.7 of Abramowitz& Stegun
                frac = Kappa/Nu; r = sqrt(1+frac.^2);
                LogNormalize  = -Nu*(r-log(1+r))  + 0.25*log(Kappa.^2+Nu.^2);
            end
            if ~equalP
                LogNormalize = LogNormalize + log(P);
            end
            MuKappa = bsxfun(@times, Mu, Kappa);
            weight = bsxfun(@plus, X.' * MuKappa, LogNormalize);
            if ~hardcluster
                weight = exp(bsxfun(@minus, weight, logsumexp(weight, 2)));
            end
            if ~equalP
                if hardcluster
                    P = countLabel / N;
                else
                    P = sum(weight, 1)/ N;
                end
            end
            [unused, L] = max(weight,[],2); %#ok<ASGLU>
            L = L.';
        end
    end
    if hardcluster && equalKappa && equalP
        Rbar = sum(Rbar) /N;
        Kappa = (Rbar*Dim - Rbar.^3)./(1-Rbar.^2);
    end
    %fprintf('\n');
end